# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import os
import math
import xlrd
import pandas as pd

def sample_pretreat(filepath,sample,fraction,result_dir,file_class_dics,file_fraction_dics):
	file=open(filepath,'r')
	file.readline()
	i=0
	j=0
	XICs={'time':[],'mz':[],'charge':[],'intensity':[],'RT_length':[],'points':[],'scans':[],'peaks':[]}
	while True: 
		oneline=file.readline()
		if not oneline:
			break
		n=1
		while j<len(oneline):
			if oneline[j]=='\t' or j==len(oneline)-1:
				if n==1:
					file_name=oneline[i:j]
					file_sample=file_class_dics[file_name]
					if not file_sample==sample:
						try:
							df=pd.DataFrame(XICs)
						except:
							print(filepath+' is not complete!')
							erro_files=open('erro_files.txt','a')
							erro_files.write(filepath+'\n')
							erro_files.close()
							return False
						df=pd.DataFrame(XICs)
						drop_negative=df[df['charge']==1].index
						df.drop(drop_negative,inplace=True)
						Tmz=[str(round(a,3))for a in df['mz']]
						df.loc[:,'Tmz']=Tmz
						sum_of_intensity=df['intensity'].sum()
						Tintensity=[math.log2(a)for a in df['intensity']]
						Tintensity10=[math.log10(a)for a in df['intensity']]
						df.loc[:,'Tintensity']=Tintensity
						Tmass=[str(round(c*float(d)-c*1.007276,2)) for c,d in zip(df['charge'],df['Tmz'])]
						df.loc[:,'Tmass']=Tmass
						df.loc[:,'Tintensity10']=Tintensity10
						Pintensity=[a/sum_of_intensity for a in df['intensity']]
						df.loc[:,'Pintensity']=Pintensity
						df.loc[:,'rt_start']=df['time']-df['RT_length']/2
						df.loc[:,'rt_end']=df['time']+df['RT_length']/2
						drop_zero=df[df['Tintensity']<=0].index
						df.drop(drop_zero,inplace=True)
						if not os.path.exists(result_dir):
							os.mkdir(result_dir)
						if not os.path.exists(result_dir+'/'+fraction):
							os.mkdir(result_dir+'/'+fraction)
						df.to_csv(result_dir+'/'+fraction+'/'+sample+'.csv',index=False)
						
						sample=file_sample
						fraction=file_fraction_dics[file_name]
						XICs={'time':[],'mz':[],'charge':[],'intensity':[],'RT_length':[],'points':[],'scans':[],'peaks':[]}
						
					n=n+1
					j=j+1
					i=j
					continue
				if n==3:
					XICs['charge'].append(int(oneline[i:j]))
					n=n+1
					j=j+1
					i=j
					continue
				if n==4:
					XICs['mz'].append(float(oneline[i:j]))
					n=n+1
					j=j+1
					i=j
					continue
				if n==8:
					XICs['points'].append(int(oneline[i:j]))
					n=n+1
					j=j+1
					i=j
					continue
				if n==9:
					XICs['scans'].append(int(oneline[i:j]))
					n=n+1
					j=j+1
					i=j
					continue
				if n==10:
					XICs['peaks'].append(int(oneline[i:j]))
					n=n+1
					j=j+1
					i=j
					continue
				if n==16:
					XICs['time'].append(float(oneline[i:j]))
					n=n+1
					j=j+1
					i=j
					continue
				if n==17:
					XICs['RT_length'].append(float(oneline[i:j])/60)
					n=n+1
					j=j+1
					i=j
					continue
				if n==31:
					XICs['intensity'].append(float(oneline[i:j]))
					i=0
					j=0
					break
				n=n+1
				j=j+1
				i=j
			else:
				j=j+1
	try:
		df=pd.DataFrame(XICs)
	except:
		print(filepath+' is not complete!')
		erro_files=open('erro_files.txt','a')
		erro_files.write(filepath+'\n')
		erro_files.close()
		return False
	df=pd.DataFrame(XICs)
	drop_negative=df[df['charge']==1].index
	df.drop(drop_negative,inplace=True)
	Tmz=[str(round(a,3))for a in df['mz']]
	df.loc[:,'Tmz']=Tmz
	sum_of_intensity=df['intensity'].sum()
	Tintensity=[math.log2(a)for a in df['intensity']]
	Tintensity10=[math.log10(a)for a in df['intensity']]
	df.loc[:,'Tintensity']=Tintensity
	Tmass=[str(round(c*float(d)-c*1.007276,2)) for c,d in zip(df['charge'],df['Tmz'])]
	df.loc[:,'Tmass']=Tmass
	df.loc[:,'Tintensity10']=Tintensity10
	Pintensity=[a/sum_of_intensity for a in df['intensity']]
	df.loc[:,'Pintensity']=Pintensity
	df.loc[:,'rt_start']=df['time']-df['RT_length']/2
	df.loc[:,'rt_end']=df['time']+df['RT_length']/2
	drop_zero=df[df['Tintensity']<=0].index
	df.drop(drop_zero,inplace=True)
	if not os.path.exists(result_dir):
		os.mkdir(result_dir)
	if not os.path.exists(result_dir+'/'+fraction):
		os.mkdir(result_dir+'/'+fraction)
	df.to_csv(result_dir+'/'+fraction+'/'+sample+'.csv',index=False)
	return True


def pre_step(file_dir,sample_file):
	workbook = xlrd.open_workbook(sample_file)
	booksheet = workbook.sheet_by_index(0)
	file_class_dics = {}
	file_fraction_dics = {}
	rows = booksheet.nrows
	for i in range(rows):
		raw_name = booksheet.cell_value(i,0).split('.')[0]
		sample_name = booksheet.cell_value(i,1)
		fraction_name = booksheet.cell_value(i,2)
		file_class_dics[raw_name] = sample_name
		file_fraction_dics[raw_name] = fraction_name
	result_dir='pre_result'
	for raw_file in os.listdir(file_dir):
		print('step_1:',raw_file)
		file=open(file_dir+'/'+raw_file,'r')
		oneline=file.readline()
		oneline=file.readline()
		file_begin=oneline.split('\t')[0]
		sample=file_class_dics[file_begin]
		fraction=file_fraction_dics[file_begin]
		file.close()
		sample_pretreat(file_dir+'/'+raw_file,sample,fraction,result_dir,file_class_dics,file_fraction_dics)



# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import pandas as pd
import os

def mass_filter():
	file_dir='shift_result_bins'
	result_dir='shift_result_bins_filter'
	
	if not os.path.exists(result_dir):
		os.mkdir(result_dir)
	
	total_number=len(os.listdir(file_dir))
	n=0
	for file in os.listdir(file_dir):
		n=n+1
		print('step_4:',n,file,total_number)
		df=pd.read_csv(file_dir+'/'+file,converters={'Tmass':str})
		grouped=df.groupby(['sample','fraction'])
		for name,group in grouped:
			group_sort=group.sort_values(by='intensity',ascending=False).copy()
			while len(group_sort)>0:
				m=0
				for index in group_sort.index:
					if m==0:
						index_begin=index
						time_begin=group_sort.loc[index_begin]['Ttime']
						m=m+1
						group_sort.drop(index,axis=0,inplace=True)
					else:
						time=group_sort.loc[index]['Ttime']
						if abs(time-time_begin)<1:
							m=m+1
							group_sort.drop(index,axis=0,inplace=True)
							df.drop(index,axis=0,inplace=True)
		df.to_csv(result_dir+'/'+file,index=False)


# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

from deeprtalign import collect_time_shift
from deeprtalign import collect_mass_alignment
import argparse
import shutil

def run(method,file_dir,sample_file,processing_number=1,percent=0.2,bin_width=0.03,bin_precision=2,dict_size=1024):
	if method=='Dinosaur':
		import deeprtalign.pre_step.dinosaur
		deeprtalign.pre_step.dinosaur.pre_step(file_dir,sample_file)
	if method=='XICFinder':
		import deeprtalign.pre_step.xicfinder
		deeprtalign.pre_step.xicfinder.pre_step(file_dir,sample_file)
	if method=='OpenMS':
		import deeprtalign.pre_step.openms
		deeprtalign.pre_step.openms.pre_step(file_dir,sample_file)
	if method=='MaxQuant':
		import deeprtalign.pre_step.maxquant
		deeprtalign.pre_step.maxquant.pre_step(file_dir,sample_file)
	collect_time_shift.collect_shift()
	if method=='Dinosaur':
		import deeprtalign.collect_mass_information_shift_bins.dinosaur
		deeprtalign.collect_mass_information_shift_bins.dinosaur.collect_bins(bin_width,bin_precision,dict_size)
	if method=='XICFinder':
		import deeprtalign.collect_mass_information_shift_bins.xicfinder
		deeprtalign.collect_mass_information_shift_bins.xicfinder.collect_bins(bin_width,bin_precision,dict_size)
	if method=='OpenMS':
		import deeprtalign.collect_mass_information_shift_bins.openms
		deeprtalign.collect_mass_information_shift_bins.openms.collect_bins(bin_width,bin_precision,dict_size)
	if method=='MaxQuant':
		import deeprtalign.collect_mass_information_shift_bins.maxquant
		deeprtalign.collect_mass_information_shift_bins.maxquant.collect_bins(bin_width,bin_precision,dict_size)
	if processing_number>1:
		from deeprtalign import mass_information_filter_multi
		mass_information_filter_multi.run_mass_filter(processing_number)
	else:
		from deeprtalign import mass_information_filter
		mass_information_filter.mass_filter()
	if processing_number>1:
		from deeprtalign import mass_alignment_samples_multi
		mass_alignment_samples_multi.run_alignment(processing_number,percent)
	else:
		from deeprtalign import mass_alignment_samples
		mass_alignment_samples.run_alignment(percent)
	collect_mass_alignment.collect_information(bin_precision,bin_width,percent)
	shutil.rmtree('pre_result')
	shutil.rmtree('shift_result')
	shutil.rmtree('shift_result_bins')
	shutil.rmtree('shift_result_bins_filter')
	shutil.rmtree('mass_align_all')
	
parser = argparse.ArgumentParser()
parser.add_argument('--method', '-m', type=str, help='the feature extraction method, support Dinosaur, XICFinder, OpenMS and MaxQuant',required=True,choices=['Dinosaur','XICFinder','OpenMS','MaxQuant'])
parser.add_argument('--file_dir', '-f', type=str, help='the data folder', required=True)
parser.add_argument('--sample_file', '-s', type=str, help='the sample file', required=True)
parser.add_argument('--processing_number', '-pn', type=int, help='processing number, choose according to the number of CPUs', default=1)
parser.add_argument('--percent', '-pt', type=float, help='skip the bins with sample numbers below the percent of total sample numbers ', default=0.5)
parser.add_argument('--bin_width', '-bw', type=float, help='the bin width, choose according to the feature extraction step', default=0.03)
parser.add_argument('--bin_precision', '-bp', type=int, help='the decimal place of bins, choose according to the feature extraction step', default=2)
parser.add_argument('--dict_size', '-ds', type=int, help='the dict size, choose according to the memory size', default=1024)
args = parser.parse_args()
	
if __name__ == '__main__':
	run(args.method,args.file_dir,args.sample_file,args.processing_number,args.percent,args.bin_width,args.bin_precision,args.dict_size)


# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import pandas as pd
import os

def collect_shift():
	folder='pre_result'
	result_folder='shift_result'
	
	fraction_1=os.listdir(folder)[0]
	anchor_sample=os.listdir(folder+'/'+fraction_1)[0]
	
	time_window=1
	
	if not os.path.exists(result_folder):
		os.mkdir(result_folder)
	
	for fraction in os.listdir(folder):
		if not os.path.exists(result_folder+'/'+fraction):
			os.mkdir(result_folder+'/'+fraction)
		anchor=pd.read_csv(folder+'/'+fraction+'/'+anchor_sample,converters={'Tmass':str,'Tmz':str})
		anchor.sort_values(by='time',ascending=True,inplace=True)
		time_total=anchor.iloc[-1]['time']-anchor.iloc[0]['time']
		anchor['Ttime']=anchor['time'].apply(lambda x:x/time_total*80)
		anchor.to_csv(result_folder+'/'+fraction+'/'+anchor_sample,index=False)
		anchor.sort_values(by='intensity',ascending=False,inplace=True)
		anchor.drop_duplicates(['Tmass'],keep='first',inplace=True)
		for file in os.listdir(folder+'/'+fraction):
			if file==anchor_sample:
				continue
			else:
				print('step_2:',fraction,file)
				result=pd.DataFrame()
				sample=pd.read_csv(folder+'/'+fraction+'/'+file,converters={'Tmass':str,'Tmz':str})
				sample.sort_values(by='time',ascending=True,inplace=True)
				time_total=sample.iloc[-1]['time']-sample.iloc[0]['time']
				sample['Ttime']=sample['time'].apply(lambda x:x/time_total*80)
				sample_sort=sample.sort_values(by='intensity',ascending=False)
				sample_drop=sample_sort.drop_duplicates(['Tmass'],keep='first')
				df = pd.merge(anchor, sample_drop, on=['Tmass'], how='inner')
				df['shift']=df.apply(lambda x:int(x['Ttime_x']-x['Ttime_y']),axis=1)
				df.sort_values(by='Ttime_y',ascending=True,inplace=True)
				time_begin=int(df.iloc[0]['Ttime_y'])
				time_end=int(df.iloc[-1]['Ttime_y'])+time_window
				time_1=time_begin-1
				time_2=time_1+time_window
				average_time_shift=0
				while time_2<=time_end:
					df_12=df[(df['Ttime_y']>=time_1)&(df['Ttime_y']<time_2)]
					if not len(df_12)==0:
						mode_time=df_12['shift'].mode().iloc[0]
						df_12=df[(df['shift']>mode_time-5)&(df['shift']<mode_time+5)]
						average_time_shift=df_12['shift'].mean(axis=0)
					sample_12=sample[(sample['Ttime']>=time_1)&(sample['Ttime']<time_2)].copy()
					if len(sample_12)==0:
						time_1=time_1+time_window
						time_2=time_2+time_window
						continue
					sample_12['Ttime']=sample_12.apply(lambda x:x['Ttime']+average_time_shift,axis=1)
					if len(result)==0:
						result=sample_12
					else:
						result=pd.concat([result,sample_12],ignore_index=True,sort=False)
					time_1=time_1+time_window
					time_2=time_2+time_window
				result.to_csv(result_folder+'/'+fraction+'/'+file,index=False)


# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import torch
import torch.nn as nn
import torch.nn.functional as F
from torch import optim
import torch.utils.data as Data
import numpy as np
import pandas as pd
import os
import multiprocessing as mp
import pkg_resources

class MatchingNetwork(nn.Module):
	def __init__(self):
		super().__init__()
		self.linear1 = nn.Sequential(
			nn.Linear(40, 5000),
			nn.Sigmoid(),
			
			nn.Linear(5000, 1000),
			nn.Sigmoid(),

			nn.Linear(1000, 50),
			nn.Sigmoid()
			)

		self.fc1 = nn.Sequential(
			nn.Linear(50, 1),
			nn.Sigmoid()
			)

	def forward(self, x):
		output = self.linear1(x)
		output = self.fc1(output)
		
		return output

def get_np_from_df(df,dimension,intensity,time):
	mass_np=np.empty((dimension,2))
	df=df.sort_values(by='Ttime',ascending=True)
	df.reset_index(drop=True,inplace=True)
	mass_df=df[df['Tintensity']==intensity]
	best_index=mass_df.index[0]
	if not len(mass_df)==1:
		time_diff_best=10000
		for index in mass_df.index:
			mass_time=mass_df.loc[index]['Ttime']
			time_diff=abs(mass_time-time)
			if time_diff<time_diff_best:
				time_diff_best=time_diff
				best_index=index
	middle_dimension=dimension//2
	for m in range(middle_dimension+1):
		if best_index-m>=0:
			intensity=df.loc[best_index-m]['Tintensity']
			time=df.loc[best_index-m]['Ttime']
			mass_np[middle_dimension-m]=np.array([intensity,time])
		else:
			mass_np[middle_dimension-m]=np.array([0,0])
		if best_index+m<len(df):
			intensity=df.loc[best_index+m]['Tintensity']
			time=df.loc[best_index+m]['Ttime']
			mass_np[middle_dimension+m]=np.array([intensity,time])
		else:
			mass_np[middle_dimension+m]=np.array([0,0])
	return mass_np

def mass_alignment(mass_folder,file,result_folder,total_sample_number,percent):
	params_file = pkg_resources.resource_filename('deeprtalign', 'data/params.pt')
	base_file = pkg_resources.resource_filename('deeprtalign', 'data/base.npy')
	net = MatchingNetwork()
	net.load_state_dict(torch.load(params_file,map_location=torch.device('cpu')))
	
	dimension=5
	
	base_1=np.load(base_file)
	base=base_1[0][2]
	
	mass_df=pd.read_csv(mass_folder+'/'+file,converters={'Tmass':str})

	sample_list=mass_df['sample'].value_counts()
	if len(sample_list)<(total_sample_number*percent):
		return 0
	sample_df=sample_list.to_frame(name='number')
	sample_df.sort_values(by='number',ascending=False,inplace=True)
	sample_df.reset_index(drop=False,inplace=True)
	sample_df.rename(columns={'index':'sample'},inplace=True)

	mass_df.sort_values(by='intensity',ascending=False,inplace=True)
	mass_df.reset_index(inplace=True,drop=True)
	mass_df['status']='unuse'
	mass_df['group']=-1
	
	sample_middle=int(len(sample_df)/2)
	sample_index=[]
	for k in range(len(sample_df)):
		if k==0:
			sample_index.append(sample_middle)
			continue
		if sample_middle-k>=0:
			sample_index.append(sample_middle-k)
		if sample_middle+k<len(sample_df):
			sample_index.append(sample_middle+k)
	sample_df=sample_df.reindex(sample_index)
	sample_df.reset_index(drop=True,inplace=True)
	
	group=0
	
	for sample_begin_index in sample_df.index:
		if sample_begin_index+1>= len(sample_df):
			break
		begin_sample=sample_df.loc[sample_begin_index]['sample']
		begin_sample_mass_df=mass_df[(mass_df['sample']==begin_sample)&(mass_df['status']=='unuse')]
		m=0
		for begin_index in begin_sample_mass_df.index:
			m=m+1
			mass_df.loc[begin_index,'status']='use'
			mass_df.loc[begin_index,'group']=group
			mass=begin_sample_mass_df.loc[begin_index]['Tmass']
			sample=begin_sample_mass_df.loc[begin_index]['sample']
			intensity=begin_sample_mass_df.loc[begin_index]['Tintensity']
			begin_time=begin_sample_mass_df.loc[begin_index]['Ttime']
			begin_charge=begin_sample_mass_df.loc[begin_index]['charge']
			fraction=int(begin_sample_mass_df.loc[begin_index]['fraction'].split('F')[-1])
			mass_df_sample=mass_df[(mass_df['sample']==sample)&(mass_df['fraction']=='F'+str(fraction))]
			mass_np_1=get_np_from_df(mass_df_sample,dimension,intensity,begin_time)
			mass_np_1=(mass_np_1-base)/base
			n=1
			while sample_begin_index+n< len(sample_df):
				next_sample=sample_df.loc[sample_begin_index+n]['sample']
				next_sample_mass_df=mass_df[(mass_df['sample']==next_sample)&(mass_df['status']=='unuse')]
				next_sample_mass_df=next_sample_mass_df[next_sample_mass_df['fraction']=='F'+str(fraction)]
				best_score=0
				best_next_index=-1
				for next_index in next_sample_mass_df.index:
					mass=next_sample_mass_df.loc[next_index]['Tmass']
					sample=next_sample_mass_df.loc[next_index]['sample']
					intensity=next_sample_mass_df.loc[next_index]['Tintensity']
					time=next_sample_mass_df.loc[next_index]['Ttime']
					next_charge=next_sample_mass_df.loc[next_index]['charge']
					if abs(begin_time-time)>5:
						continue
					fraction=int(next_sample_mass_df.loc[next_index]['fraction'].split('F')[-1])
					mass_df_sample=mass_df[(mass_df['sample']==sample)&(mass_df['fraction']=='F'+str(fraction))]
					mass_np_2=get_np_from_df(mass_df_sample,dimension,intensity,time)
					mass_np_2=(mass_np_2-base)/base
					cycle_1=np.expand_dims(mass_np_1[2],0).repeat(5,axis=0)-mass_np_2
					cycle_1=cycle_1.reshape(10)
					cycle_2=np.expand_dims(mass_np_2[2],0).repeat(5,axis=0)-mass_np_1
					cycle_2=cycle_2.reshape(10)
					re_mass_np_1=mass_np_1.reshape(10)
					re_mass_np_2=mass_np_2.reshape(10)
					cycle=np.hstack((cycle_1,cycle_2,re_mass_np_1,re_mass_np_2))
					cycle=torch.tensor(cycle,dtype=torch.float32)
					output = net(cycle)
					output = 1-output[0].tolist()
					if (output > best_score) and (output > 0.5):
						best_next_index=next_index
						break
				if best_next_index==-1:
					n=n+1
					continue
				mass_df.loc[best_next_index,'status']='use'
				mass_df.loc[best_next_index,'group']=group
				n=n+1
			group=group+1
	mass_df.to_csv(result_folder+'/'+file,index=False)
	return 1
	
def run_alignment(processing_number,percent):
	mass_folder='shift_result_bins_filter'
	result_folder='mass_align_all'

	fraction_1=os.listdir('pre_result')[0]
	total_sample_number=len(os.listdir('pre_result/'+fraction_1))
	
	
	if not os.path.exists(result_folder):
		os.mkdir(result_folder)
	
	pool_arg=[]
	for file in os.listdir(mass_folder):
		file_arg=[]
		file_arg.append(mass_folder)
		file_arg.append(file)
		file_arg.append(result_folder)
		file_arg.append(total_sample_number)
		file_arg.append(percent)
		pool_arg.append(file_arg)
	print('step_5: running')
	pool=mp.Pool(processes=processing_number)
	result = pool.starmap(mass_alignment,pool_arg)
	pool.close()
	pool.join()
	print('step_5: finish')


# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import pandas as pd
import multiprocessing as mp
import os

def mass_filter(file_dir,file,result_dir):
	df=pd.read_csv(file_dir+'/'+file,converters={'Tmass':str})
	grouped=df.groupby(['sample','fraction'])
	for name,group in grouped:
		group_sort=group.sort_values(by='intensity',ascending=False).copy()
		while len(group_sort)>0:
			m=0
			for index in group_sort.index:
				if m==0:
					index_begin=index
					time_begin=group_sort.loc[index_begin]['Ttime']
					m=m+1
					group_sort.drop(index,axis=0,inplace=True)
				else:
					time=group_sort.loc[index]['Ttime']
					if abs(time-time_begin)<1:
						m=m+1
						group_sort.drop(index,axis=0,inplace=True)
						df.drop(index,axis=0,inplace=True)
	df.to_csv(result_dir+'/'+file,index=False)
def run_mass_filter(processing_number):
	file_dir='shift_result_bins'
	result_dir='shift_result_bins_filter'
	
	if not os.path.exists(result_dir):
		os.mkdir(result_dir)
	
	pool_arg=[]
	for file in os.listdir(file_dir):
		file_arg=[]
		file_arg.append(file_dir)
		file_arg.append(file)
		file_arg.append(result_dir)
		pool_arg.append(file_arg)
	print('step_4: running')
	pool=mp.Pool(processes=processing_number)
	result = pool.starmap(mass_filter,pool_arg)
	pool.close()
	pool.join()
	print('step_4: finish')


# -*- coding: utf-8 -*-
"""
Created on Sun Apr 25 16:28:49 2021

@author: leoicarus
"""
import pandas as pd
import os
import shutil
import tracemalloc

def collect_bins(bin_width,bin_precision,dict_size):
	file_dir='shift_result'
	
	result_dir='shift_result_bins'
	
	bin_df=pd.DataFrame()
	
	if not os.path.exists(result_dir):
		os.mkdir(result_dir)
	
	shift_result={}
	tracemalloc.start()
	for fraction in os.listdir(file_dir):
		for file in os.listdir(file_dir+'/'+fraction):
			print('step_3:',file,fraction)
			file_path=file_dir+'/'+fraction+'/'+file
			sample=file.split('.csv')[0]
			if len(bin_df)==0:
				df=pd.read_csv(file_path)
				df.sort_values(by='Tmass',ascending=True,inplace=True)
				bin_begin=df.iloc[0]['Tmass']
				bin_end=df.iloc[-1]['Tmass']
				bin_1=bin_begin-bin_width
				bin_2=bin_begin+bin_width
				bin_id=0
				bin_list={'bin_id':[],'bin_1':[],'bin_2':[]}
				while bin_2<bin_end:
					bin_list['bin_id'].append(bin_id)
					bin_list['bin_1'].append(str(round(bin_1,bin_precision)))
					bin_list['bin_2'].append(str(round(bin_2,bin_precision)))
					bin_1=bin_1+bin_width/2
					bin_2=bin_1+bin_width
					bin_id=bin_id+1
				if bin_1<bin_end:
					bin_2=bin_end+bin_width
					bin_list['bin_id'].append(bin_id)
					bin_list['bin_1'].append(str(round(bin_1,bin_precision)))
					bin_list['bin_2'].append(str(round(bin_2,bin_precision)))
				bin_df=pd.DataFrame(bin_list)
				bin_1_f=[float(a)for a in bin_df['bin_1']]
				bin_2_f=[float(a)for a in bin_df['bin_2']]
				bin_df['bin_1_f']=bin_1_f
				bin_df['bin_2_f']=bin_2_f
				bin_number=len(bin_df)
				batch_size=int(bin_number/50)
			batch_number=0
			bin_index_begin=0
			while bin_index_begin+batch_size<bin_number:
				bin_df.loc[bin_index_begin:bin_index_begin+batch_size,'batch_number']=int(batch_number)
				bin_index_begin=bin_index_begin+batch_size+1
				batch_number=batch_number+1
			bin_df.loc[bin_index_begin:bin_number,'batch_number']=int(batch_number)
			file=open(file_path,'r')
			oneline=file.readline()
			title_line='sample,fraction,'+oneline
			while oneline:
				oneline=file.readline()
				if not oneline:
					break
				mass=oneline.split(',')[22].strip()
				bin_mass=bin_df[(bin_df['bin_1_f']<=float(mass))&(bin_df['bin_2_f']>float(mass))]
				if len(bin_mass)==0:
					continue
				for index in bin_mass.index:
					bin_1=bin_mass.loc[index]['bin_1']
					bin_2=bin_mass.loc[index]['bin_2']
					batch_number=bin_mass.loc[index]['batch_number']
					new_key=result_dir+'/'+str(batch_number)+'/'+bin_1+'_'+bin_2+'.csv'
					if new_key in shift_result.keys():
						shift_result[new_key]=shift_result[new_key]+sample+','+fraction+','+oneline
					else:
						shift_result[new_key]=sample+','+fraction+','+oneline
					current, peak = tracemalloc.get_traced_memory()
					if current>dict_size*1024*1024:
						for key in shift_result:
							if not os.path.exists(key.split('/')[0]+'/'+key.split('/')[1]):
								os.mkdir(result_dir+'/'+str(batch_number))
							if not os.path.exists(key):
								result_file=open(key,'a')
								result_file.write(title_line)
								result_file.close()
							result_file=open(key,'a')
							result_file.write(shift_result[key])
							result_file.close()
						shift_result={}
	for key in shift_result:
		if not os.path.exists(key.split('/')[0]+'/'+key.split('/')[1]):
			os.mkdir(key.split('/')[0]+'/'+key.split('/')[1])
		if not os.path.exists(key):
			result_file=open(key,'a')
			result_file.write(title_line)
			result_file.close()
		result_file=open(key,'a')
		result_file.write(shift_result[key])
		result_file.close()
	shift_result={}
	tracemalloc.stop()
	for son_path in os.listdir(result_dir):
		dir_path=os.path.join(result_dir,son_path)
		if os.path.isdir(dir_path):
			print(dir_path)
			for file in os.listdir(dir_path):
				file = os.path.join(dir_path, file)
				shutil.move(file, result_dir)
			os.rmdir(dir_path)




# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import os
import math
import xlrd
import pandas as pd

def sample_pretreat(filepath,sample,fraction,result_dir):
	df=pd.read_csv(filepath,sep='\t')
	drop_negative=df[df['charge']==1].index
	df.drop(drop_negative,inplace=True)
	Tmz=[str(round(a,3))for a in df['mz']]
	df.loc[:,'Tmz']=Tmz
	df.loc[:,'intensity']=df['intensitySum']
	df.loc[:,'rt_start']=df['rtStart']
	df.loc[:,'time']=df['rtApex']
	df.loc[:,'rt_end']=df['rtEnd']
	sum_of_intensity=df['intensity'].sum()
	Tintensity=[math.log2(a)for a in df['intensity']]
	Tintensity10=[math.log10(a)for a in df['intensity']]
	df.loc[:,'Tintensity']=Tintensity
	df.loc[:,'Tintensity10']=Tintensity10
	Pintensity=[a/sum_of_intensity for a in df['intensity']]
	df.loc[:,'Pintensity']=Pintensity
	drop_zero=df[df['Tintensity']<=0].index
	df.drop(drop_zero,inplace=True)
	Tmass=[str(round(c*float(d)-c*1.007276,2)) for c,d in zip(df['charge'],df['Tmz'])]
	df.loc[:,'Tmass']=Tmass
	if not os.path.exists(result_dir):
		os.mkdir(result_dir)
	if not os.path.exists(result_dir+'/'+fraction):
		os.mkdir(result_dir+'/'+fraction)
	df.to_csv(result_dir+'/'+fraction+'/'+sample+'.csv',index=False)
	return True


def pre_step(file_dir,sample_file):
	workbook = xlrd.open_workbook(sample_file)
	booksheet = workbook.sheet_by_index(0)
	file_class_dics = {}
	file_fraction_dics = {}
	rows = booksheet.nrows
	for i in range(rows):
		raw_name = booksheet.cell_value(i,0).split('.')[0]
		sample_name = booksheet.cell_value(i,1)
		fraction_name = booksheet.cell_value(i,2)
		file_class_dics[raw_name] = sample_name
		file_fraction_dics[raw_name] = fraction_name
	result_dir='pre_result'
	for file in os.listdir(file_dir):
		if not file.split('.')[0] in file_class_dics.keys():
				print('file not in list!')
				continue
		print('step_1:',file)
		sample=file_class_dics[file.split('.')[0]]
		fraction=file_fraction_dics[file.split('.')[0]]
		sample_pretreat(file_dir+'/'+file,sample,fraction,result_dir)



from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
	name='deeprtalign',
	version="1.0.7",
	packages=find_packages(),
	python_requires='>=3.4',
	install_requires=[
						'xlrd==1.2.0',
						'pandas>=0.24',
		],

	package_data={
		'deeprtalign': ['data/base.npy','data/params.pt']
		},

	author='Yi Liu',
	author_email='leoicarus@163.com',
	description='retention time alignment tool for large cohort LC-MS data analysis',
	long_description=long_description,
	long_description_content_type='text/markdown',
	url='https://github.com/FineLiu/deeprtalign',
	license='GPLv3'
)


# -*- coding: utf-8 -*-
# Copyright © 2021 Yi Liu and Cheng Chang
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the included LICENSE file for details.

import pandas as pd
import numpy as np
import math
import os

def collect_information(bin_precision,bin_width,percent):
	folder='mass_align_all'
	
	fraction_1=os.listdir('pre_result')[0]
	total_sample_number=len(os.listdir('pre_result/'+fraction_1))
	
	result_folder='mass_align_all_information'
	if not os.path.exists(result_folder):
		os.mkdir(result_folder)
	
	
	result_df=pd.DataFrame()
	total_number=len(os.listdir(folder))
	n=0
	m=0
	k=1
	for file in os.listdir(folder):
		n=n+1
		m=m+1
		file_path=folder+'/'+file
		window=file.split('.csv')[0]
		window_1=round(float(window.split('_')[0]),bin_precision)
		window_list=[]
		window_size=0
		while window_size<bin_width:
			window_2=round(window_1+window_size,bin_precision)
			window_list.append(window_2)
			window_size=round(window_size+1/math.pow(10,bin_precision),2)
		window_central=str(round(np.median(window_list),bin_precision))
		print('step_6:',n,window,total_number)
		file_df=pd.read_csv(file_path,converters={'Tmass':str})
		file_df=file_df[file_df['status']=='use']
		grouped=file_df.groupby(['group'])
		for name,group in grouped:
			if len(group)<total_sample_number*percent:
				continue
			group_central=group[group['Tmass']==window_central]
			if len(group_central)<len(group)/2:
				continue
			window_group=window+'_'+str(name)
			group['group']=window_group
			if len(result_df)==0:
				result_df=group
			else:
				result_df=pd.concat([result_df,group],ignore_index=True,sort=False)
		if total_number>10000:
			if m>10000:
				result_df.to_csv(result_folder+'/'+'information_'+str(k)+'.csv',index=False)
				result_df=pd.DataFrame()
				k=k+1
				m=0
	result_df.to_csv(result_folder+'/'+'information_'+str(k)+'.csv',index=False)